import numpy as np
import tkinter as tk
from tkinter import font  as tkfont
from tkinter import filedialog
from tkinter import ttk
from tkinter import *
import json
import cv2
from getCoordinates import findWellLeft, findWellRight, findHeadCenter, findBodyExtremity
import math
from findWells import findWells
from getHyperparameters import getHyperparametersSimple
from getBackground import getBackground
from getForegroundImage import getForegroundImage
import cvui
from vars import getGlobalVariables
import json
import os
globalVariables = getGlobalVariables()

def chooseVideoToCreateConfigFileFor(self, controller, reloadConfigFile):

  if int(reloadConfigFile):
    if globalVariables["mac"]:
      configFileName =  filedialog.askopenfilename(initialdir = "./configuration/",title = "Select file")
    else:
      configFileName =  filedialog.askopenfilename(initialdir = "./configuration/",title = "Select file",filetypes = (("json files","*.json"),("all files","*.*")))
    with open(configFileName) as f:
      self.configFile = json.load(f)

  if globalVariables["mac"]:
    self.videoToCreateConfigFileFor = filedialog.askopenfilename(initialdir = os.path.expanduser("~"),title = "Select video to create config file for")
  else:
    self.videoToCreateConfigFileFor = filedialog.askopenfilename(initialdir = os.path.expanduser("~"),title = "Select video to create config file for",filetypes = (("video","*.*"),("all files","*.*")))
  controller.show_frame("ChooseGeneralExperiment")

def chooseGeneralExperiment(self, controller, freeZebra, headEmbZebra, drosophilia, rodent, other):
  self.configFile["extractAdvanceZebraParameters"] = 0
  if int(freeZebra):
    self.organism = 'zebrafish'
    self.configFile["headEmbeded"] = 0
    controller.show_frame("WellOrganisation")
  elif int(headEmbZebra):
    self.organism = 'headembeddedzebrafish'
    self.configFile["headEmbeded"] = 1
    chooseBeginningAndEndOfVideo(self, controller)
  else:
    self.organism = 'drosoorrodent'
    self.configFile["headEmbeded"] = 0
    self.configFile["freeSwimmingTailTrackingMethod"] = "none"
    controller.show_frame("WellOrganisation")

def wellOrganisation(self, controller, circular, roi, other):
  if circular and self.organism != 'drosoorrodent':
    controller.show_frame("CircularWells")
  else:
    if roi:
      cap = cv2.VideoCapture(self.videoToCreateConfigFileFor)
      cap.set(1, 10)
      ret, frame = cap.read()
      
      WINDOW_NAME = "Click on the top left of the region of interest"
      cvui.init(WINDOW_NAME)
      cv2.moveWindow(WINDOW_NAME, 0,0)
      cvui.imshow(WINDOW_NAME, frame)
      while not(cvui.mouse(cvui.CLICK)):
        cursor = cvui.mouse()
        if cv2.waitKey(20) == 27:
          break
      self.configFile["oneWellManuallyChosenTopLeft"] = [cursor.x, cursor.y]
      cv2.destroyAllWindows()
      
      WINDOW_NAME = "Click on the bottom right of the region of interest"
      cvui.init(WINDOW_NAME)
      cv2.moveWindow(WINDOW_NAME, 0,0)
      cvui.imshow(WINDOW_NAME, frame)
      while not(cvui.mouse(cvui.CLICK)):
        cursor = cvui.mouse()
        if cv2.waitKey(20) == 27:
          break
      self.configFile["oneWellManuallyChosenBottomRight"] = [cursor.x, cursor.y]
      cv2.destroyAllWindows()
      
      self.configFile["nbWells"] = 1
      chooseBeginningAndEndOfVideo(self, controller)
    else:
      self.configFile["noWellDetection"] = 1
      self.configFile["nbWells"] = 1
      chooseBeginningAndEndOfVideo(self, controller)

def circularWells(self, controller, nbwells, nbRowsOfWells, nbWellsPerRows):
  self.configFile["nbWells"]        = int(nbwells)
  
  if len(nbRowsOfWells):
    self.configFile["nbRowsOfWells"]  = int(nbRowsOfWells)
  else:
    self.configFile["nbRowsOfWells"]  = 1
    
  if len(nbWellsPerRows):
    self.configFile["nbWellsPerRows"]  = int(nbWellsPerRows)
  else:
    self.configFile["nbWellsPerRows"]  = 4
  
  controller.show_frame("ChooseCircularWellsLeft")

def chooseCircularWellsLeft(self, controller):
  cap = cv2.VideoCapture(self.videoToCreateConfigFileFor)
  ret, frame = cap.read()
  [x, y] = findWellLeft(frame)
  self.wellLeftBorderX = x
  self.wellLeftBorderY = y
  controller.show_frame("ChooseCircularWellsRight")
  
def chooseCircularWellsRight(self, controller):
  cap = cv2.VideoCapture(self.videoToCreateConfigFileFor)
  ret, frame = cap.read()
  [xRight, yRight] = findWellRight(frame)
  xLeft = self.wellLeftBorderX
  yLeft = self.wellLeftBorderY
  dist = math.sqrt((xLeft - xRight)**2 + (yLeft - yRight)**2)
  self.configFile["minWellDistanceForWellDetection"] = int(dist)
  self.configFile["wellOutputVideoDiameter"]         = int(dist + dist * 0.2)
  chooseBeginningAndEndOfVideo(self, controller)

def chooseBeginningAndEndOfVideo(self, controller):
  cap = cv2.VideoCapture(self.videoToCreateConfigFileFor)
  max_l = int(cap.get(7)) - 2
  
  cap.set(1, 1)
  ret, frame = cap.read()
  WINDOW_NAME = "Choose where the analysis of your video should start."
  WINDOW_NAME_CTRL = "Control"
  cvui.init(WINDOW_NAME)
  cv2.moveWindow(WINDOW_NAME, 0,0)
  cvui.init(WINDOW_NAME_CTRL)
  cv2.moveWindow(WINDOW_NAME_CTRL, 0, 300)
  value = [1]
  curValue = value[0]
  buttonclicked = False
  buttonEntireVideo = False
  widgetX = 40
  widgetY = 20
  widgetL = 300
  while not(buttonclicked) and not(buttonEntireVideo):
      value[0] = int(value[0])
      if curValue != value[0]:
        cap.set(1, value[0])
        frameOld = frame
        ret, frame = cap.read()
        if not(ret):
          frame = frameOld
        curValue = value[0]
      frameCtrl = np.full((200, 750), 100).astype('uint8')
      frameCtrl[widgetY:widgetY+60, widgetX:widgetX+widgetL] = 0
      cvui.text(frameCtrl, widgetX, widgetY, 'Frame')
      cvui.trackbar(frameCtrl, widgetX, widgetY+10, widgetL, value, 0, max_l)
      cvui.counter(frameCtrl, widgetX, widgetY+60, value)
      buttonclicked = cvui.button(frameCtrl, widgetX, widgetY+90, "Ok, I want the tracking to start at this frame!")
      
      cvui.text(frameCtrl, widgetX+350, widgetY+1, 'No, this is unecessary:')
      buttonEntireVideo = cvui.button(frameCtrl, widgetX+350, widgetY+40, "I want the tracking to run on the entire video!")
      
      cvui.text(frameCtrl, widgetX, widgetY+130, 'Keys: 4 or a: move backwards; 6 or d: move forward')
      cvui.text(frameCtrl, widgetX, widgetY+160, 'Keys: g or f: fast backwards; h or j: fast forward')
      cvui.imshow(WINDOW_NAME, frame)
      cvui.imshow(WINDOW_NAME_CTRL, frameCtrl)
      r = cv2.waitKey(20)
      if (r == 54) or (r == 100) or (r == 0):
        value[0] = value[0] + 1
      elif (r == 52) or (r == 97) or (r == 113):
        value[0] = value[0] - 1
      elif (r == 103):
        value[0] = value[0] - 30
      elif (r == 104):
        value[0] = value[0] + 30
      elif (r == 102):
        value[0] = value[0] - 100
      elif (r == 106):
        value[0] = value[0] + 100
  cv2.destroyAllWindows()
  
  if not(buttonEntireVideo):
    self.configFile["firstFrame"] = int(value[0])
    cap.set(1, max_l)
    ret, frame = cap.read()
    while not(ret):
      max_l = max_l - 1
      cap.set(1, max_l)
      ret, frame = cap.read()
    WINDOW_NAME = "Choose where the analysis of your video should end."
    WINDOW_NAME_CTRL = "Control"
    cvui.init(WINDOW_NAME)
    cv2.moveWindow(WINDOW_NAME, 0,0)
    cvui.init(WINDOW_NAME_CTRL)
    cv2.moveWindow(WINDOW_NAME_CTRL, 0, 300)
    value = [max_l]
    curValue = value[0]
    buttonclicked = False
    widgetX = 40
    widgetY = 20
    widgetL = 300
    while not(buttonclicked):
        value[0] = int(value[0])
        if curValue != value[0]:
          cap.set(1, value[0])
          frameOld = frame
          ret, frame = cap.read()
          if not(ret):
            frame = frameOld
          curValue = value[0]
        frameCtrl = np.full((200, 400), 100).astype('uint8')
        frameCtrl[widgetY:widgetY+60, widgetX:widgetX+widgetL] = 0
        cvui.text(frameCtrl, widgetX, widgetY, 'Frame')
        cvui.trackbar(frameCtrl, widgetX, widgetY+10, widgetL, value, 0, max_l-1)
        cvui.counter(frameCtrl, widgetX, widgetY+60, value)
        buttonclicked = cvui.button(frameCtrl, widgetX, widgetY+90, "Ok, I want the tracking to end at this frame!")
        cvui.text(frameCtrl, widgetX, widgetY+130, 'Keys: 4 or a: move backwards; 6 or d: move forward')
        cvui.text(frameCtrl, widgetX, widgetY+160, 'Keys: g or f: fast backwards; h or j: fast forward')
        cvui.imshow(WINDOW_NAME, frame)
        cvui.imshow(WINDOW_NAME_CTRL, frameCtrl)
        r = cv2.waitKey(20)
        if (r == 54) or (r == 100) or (r == 0):
          value[0] = value[0] + 1
        elif (r == 52) or (r == 97) or (r == 113):
          value[0] = value[0] - 1
        elif (r == 103):
          value[0] = value[0] - 30
        elif (r == 104):
          value[0] = value[0] + 30
        elif (r == 102):
          value[0] = value[0] - 100
        elif (r == 106):
          value[0] = value[0] + 100
    self.configFile["lastFrame"] = int(value[0])
    cv2.destroyAllWindows()
  
  if int(self.configFile["headEmbeded"]) == 1:
    controller.show_frame("HeadEmbeded")
  else:
    controller.show_frame("NumberOfAnimals")

  
def getImageForMultipleAnimalGUI(l, vertical, horizontal, nx, ny, max_l, videoToCreateConfigFileFor, background, wellPositions, hyperparameters):
  frame = getForegroundImage(videoToCreateConfigFileFor, background, l, 0, wellPositions, hyperparameters)
  lengthX = nx * 2
  lengthY = ny
  
  newX = lengthX
  newY = lengthY
  
  vertical2   = vertical   - vertical   * 0.12
  horizontal2 = horizontal - horizontal * 0.01
  if ( (lengthX > horizontal2) or (lengthY > vertical2) ):
    sinkFactor = 1
    sinkFactorX = horizontal2 / lengthX
    sinkFactorY = vertical2   / lengthY
    if (sinkFactorX > sinkFactorY):
      sinkFactor = sinkFactorY
    else:
      sinkFactor = sinkFactorX
    newX = lengthX * sinkFactor
    newY = lengthY * sinkFactor
  
  frame2 = frame
  ret,thresh2 = cv2.threshold(frame2,hyperparameters["thresholdForBlobImg"],255,cv2.THRESH_BINARY)
  kernel  = np.ones((hyperparameters["erodeSize"],hyperparameters["erodeSize"]), np.uint8)
  thresh2 = cv2.dilate(thresh2, kernel, iterations=hyperparameters["dilateIter"])

  thresh = thresh2
  thresh2 = cv2.cvtColor(thresh2, cv2.COLOR_GRAY2RGB)
  frame   = cv2.cvtColor(frame,   cv2.COLOR_GRAY2RGB)
  thresh[0, :]                = 255
  thresh[:, 0]                = 255
  thresh[len(thresh)-1, :]    = 255
  thresh[:, len(thresh[0])-1] = 255
  areaList = []
  contours, hierarchy = cv2.findContours(thresh,cv2.RETR_TREE,cv2.CHAIN_APPROX_SIMPLE)
  for contour in contours:
    area = cv2.contourArea(contour)
    if area < (len(thresh) * len(thresh[0]))/2:
      areaList.append(area)
    if (area > hyperparameters["minArea"]) and (area < hyperparameters["maxArea"]):
      M = cv2.moments(contour)
      if M['m00']:
        x = int(M['m10']/M['m00'])
        y = int(M['m01']/M['m00'])
        cv2.circle(thresh2, (x,y), 3, (0,0,255), -1)
      else:
        x = 0
        y = 0
  
  frame = cv2.line(frame, (len(frame[0])-5, 0), (len(frame[0])-5, len(frame)), (255, 0, 0), 5) 
  
  frame = np.concatenate((frame, thresh2), axis=1)
  
  frame = cv2.resize(frame,(int(newX),int(newY)))
  
  if len(areaList):
    maxToReturn = int((max(areaList)+2)*2)
  else:
    maxToReturn = (len(thresh) * len(thresh[0]))/5
  
  return [frame, maxToReturn]

def printStuffOnCtrlImg(frameCtrl, frameNum, x, y, l, minn, maxx, name):
  cvui.text(frameCtrl,     x,         y,    name)
  cvui.trackbar(frameCtrl, x,      y+10, l, frameNum, minn, maxx)
  cvui.counter(frameCtrl,  x+l+10, y+20,    frameNum)


def numberOfAnimals(self, controller, nbanimals, yes, noo):

  self.configFile["noBoutsDetection"] = 1
  self.configFile["noChecksForBoutSelectionInExtractParams"] = 1
  self.configFile["trackingPointSizeDisplay"] = 4
  self.configFile["validationVideoPlotHeading"] = 0
  
  nbanimals = int(nbanimals)
  if nbanimals == self.configFile["nbWells"]:
    self.configFile["nbAnimalsPerWell"] = 1
  else:
    self.configFile["nbAnimalsPerWell"] = int(nbanimals / self.configFile["nbWells"])
    if noo:
      self.configFile["multipleHeadTrackingIterativelyRelaxAreaCriteria"] = 0
    else:
      self.configFile["multipleHeadTrackingIterativelyRelaxAreaCriteria"] = 1
  
  if self.organism == 'zebrafish':
    controller.show_frame("IdentifyHeadCenter")
  else:
    tempConfig = self.configFile
    
    horizontal = self.winfo_screenwidth()
    vertical   = self.winfo_screenheight()
    
    # Wait image
    WINDOW_NAME = "Please Wait"
    cvui.init(WINDOW_NAME)
    cv2.moveWindow(WINDOW_NAME, 0,0)
    # Getting hyperparameters, wellPositions, and background
    hyperparameters = getHyperparametersSimple(tempConfig)
    wellPositions = findWells(self.videoToCreateConfigFileFor, hyperparameters)
    background    = getBackground(self.videoToCreateConfigFileFor, hyperparameters)
    
    tab = [1]
    img = cv2.imread('./code/GUI/no1.png')
    img = cv2.resize(img,(int(horizontal*0.95),int(vertical*0.8)))
    buttonclicked = False
    while not(buttonclicked):
      buttonclicked = cvui.button(img, 10, 10, "Ok, I understand!")
      cvui.imshow(WINDOW_NAME, img)
      cv2.waitKey(20)
    img = cv2.imread('./code/GUI/no2.png')
    img = cv2.resize(img,(int(horizontal*0.95),int(vertical*0.8)))
    buttonclicked = False
    while not(buttonclicked):
      buttonclicked = cvui.button(img, 10, 10, "Ok, I understand!")
      cvui.imshow(WINDOW_NAME, img)
      cv2.waitKey(20)
    img = cv2.imread('./code/GUI/ok1.png')
    img = cv2.resize(img,(int(horizontal*0.95),int(vertical*0.8)))
    buttonclicked = False
    while not(buttonclicked):
      buttonclicked = cvui.button(img, 10, 10, "Ok, I understand!")
      cvui.imshow(WINDOW_NAME, img)
      cv2.waitKey(20)
    
    WINDOW_NAME = "Adjust Parameters: As much as possible, you must see red points on and only on animals on the right image."
    WINDOW_NAME_CTRL = "Adjust Parameters."
    cv2.destroyAllWindows()
    # Manual parameters adjustements
    cap        = cv2.VideoCapture(self.videoToCreateConfigFileFor)
    nx         = int(cap.get(3))
    ny         = int(cap.get(4))
    max_l      = int(cap.get(7))
    
    hyperparameters["minArea"] = 10
    hyperparameters["maxArea"] = 800

    [frame, maxAreaBlobs] = getImageForMultipleAnimalGUI(1, vertical, horizontal, nx, ny, max_l, self.videoToCreateConfigFileFor, background, wellPositions, hyperparameters)
    frameCtrl = np.full((200, 1100), 100).astype('uint8')
    
    cvui.init(WINDOW_NAME)
    cv2.moveWindow(WINDOW_NAME, 0, 0)
    cvui.imshow(WINDOW_NAME, frame)
    
    cvui.init(WINDOW_NAME_CTRL)
    cv2.moveWindow(WINDOW_NAME_CTRL, 0, vertical-290)
    cvui.imshow(WINDOW_NAME_CTRL, frameCtrl)
    
    frameNum = [hyperparameters["firstFrame"]] if "firstFrame" in hyperparameters else [ 1 ]
    curFrameNum = frameNum[0] + 1
    minPixelDiffForBackExtract = [hyperparameters["minPixelDiffForBackExtract"]]
    thresholdForBlobImg        = [hyperparameters["thresholdForBlobImg"]]
    dilateIter                 = [hyperparameters["dilateIter"]]
    minArea                    = [hyperparameters["minArea"]]
    maxArea                    = [hyperparameters["maxArea"]]
    firstFrame = hyperparameters["firstFrame"] if "firstFrame" in hyperparameters else 1
    lastFrame  = hyperparameters["lastFrame"]-1 if "lastFrame" in hyperparameters else max_l - 10
    
    buttonclicked = False
    while not(buttonclicked):
      if curFrameNum != frameNum[0] or hyperparameters["minPixelDiffForBackExtract"] != minPixelDiffForBackExtract[0] or hyperparameters["thresholdForBlobImg"] != thresholdForBlobImg[0] or hyperparameters["dilateIter"] != dilateIter[0] or hyperparameters["minArea"] != minArea[0] or hyperparameters["maxArea"] != maxArea[0]:
        
        curFrameNum = frameNum[0]
        hyperparameters["minPixelDiffForBackExtract"] = int(minPixelDiffForBackExtract[0])
        hyperparameters["thresholdForBlobImg"] = int(thresholdForBlobImg[0])
        hyperparameters["dilateIter"] = int(dilateIter[0])
        hyperparameters["minArea"] = int(minArea[0])
        hyperparameters["maxArea"] = int(maxArea[0])
        
        [frame, maxAreaBlobs] = getImageForMultipleAnimalGUI(curFrameNum, vertical, horizontal, nx, ny, max_l, self.videoToCreateConfigFileFor, background, wellPositions, hyperparameters)
        
      frameCtrl = np.full((200, 1100), 100).astype('uint8')
      
      printStuffOnCtrlImg(frameCtrl, frameNum,                     1, 5,  350,  firstFrame, lastFrame, "Frame number")
      printStuffOnCtrlImg(frameCtrl, minPixelDiffForBackExtract, 470, 5,  350,  0, 255, "Threshold left image")
      printStuffOnCtrlImg(frameCtrl, thresholdForBlobImg,          1, 71,  350, 0, 255, "Threshold right image")
      printStuffOnCtrlImg(frameCtrl, dilateIter,                 470, 71,  350, 0, 15, "Area dilatation")
      printStuffOnCtrlImg(frameCtrl, minArea,                      1, 137, 350, 0, maxAreaBlobs, "Minimum area")
      printStuffOnCtrlImg(frameCtrl, maxArea,                    470, 137, 350, 0, maxAreaBlobs, "Maximum area")
      
      buttonclicked = cvui.button(frameCtrl, 940, 10, "Ok, done!")
      
      cvui.imshow(WINDOW_NAME, frame)
      cvui.imshow(WINDOW_NAME_CTRL, frameCtrl)
      
      if cv2.waitKey(20) == 27:
          break
    cv2.destroyAllWindows()
    
    self.configFile["minPixelDiffForBackExtract"] = int(hyperparameters["minPixelDiffForBackExtract"])
    self.configFile["thresholdForBlobImg"]        = int(hyperparameters["thresholdForBlobImg"])
    self.configFile["dilateIter"]                 = int(hyperparameters["dilateIter"])
    self.configFile["minArea"]                    = int(hyperparameters["minArea"])
    self.configFile["maxArea"]                    = int(hyperparameters["maxArea"])
    self.configFile["minAreaBody"]                = int(hyperparameters["minArea"])
    self.configFile["maxAreaBody"]                = int(hyperparameters["maxArea"])
    self.configFile["headSize"]        = math.sqrt((int(hyperparameters["minArea"]) + int(hyperparameters["maxArea"])) / 2)
    
    controller.show_frame("FinishConfig")


def chooseHeadCenter(self, controller):
  cap = cv2.VideoCapture(self.videoToCreateConfigFileFor)
  ret, frame = cap.read()
  [x, y] = findHeadCenter(frame)
  self.headCenterX = x
  self.headCenterY = y
  controller.show_frame("IdentifyBodyExtremity")

def chooseBodyExtremity(self, controller):
  cap = cv2.VideoCapture(self.videoToCreateConfigFileFor)
  ret, frame = cap.read()
  [extX, extY] = findBodyExtremity(frame)
  headCenterX = self.headCenterX
  headCenterY = self.headCenterY
  dist = math.sqrt((extX - headCenterX)**2 + (extY - headCenterY)**2)
  
  if self.organism == 'zebrafish':
    minArea = int(dist * (dist * 0.1))
    maxArea = int(dist * (dist * 0.4))
    self.configFile["minArea"]     = minArea
    self.configFile["maxArea"]     = maxArea
    self.configFile["minAreaBody"] = minArea
    self.configFile["maxAreaBody"] = maxArea
    self.configFile["headSize"]    = int(dist * 0.3)
    self.configFile["minTailSize"] = int(dist * 0.05)
    self.configFile["maxTailSize"] = int(dist * 2)
  else:
    minArea = int(((2 * dist) * (2 * dist)) * 0.2)
    maxArea = int(((2 * dist) * (2 * dist)) * 1.5)
    self.configFile["minArea"]     = minArea
    self.configFile["maxArea"]     = maxArea
    self.configFile["minAreaBody"] = minArea
    self.configFile["maxAreaBody"] = maxArea
    self.configFile["headSize"]    = int(dist * 2)
  
  self.configFile["extractBackWhiteBackground"] = 1
  
  self.configFile["noBoutsDetection"] = 1
  
  controller.show_frame("GoToAdvanceSettings")


def goToAdvanceSettings(self, controller, yes, no):

  if int(no):
    controller.show_frame("FinishConfig")
  else:
    self.configFile["noBoutsDetection"] = 0
    self.calculateBackgroundFreelySwim(controller, 0)

def finishConfig(self, controller, configFileNameToSave):
  with open('./configuration/' + configFileNameToSave + '.json', 'w') as outfile:
    json.dump(self.configFile, outfile)
  self.configFile = {}
  self.videoToCreateConfigFileFor = ''
  self.wellLeftBorderX = 0
  self.wellLeftBorderY = 0
  self.headCenterX = 0
  self.headCenterY = 0
  self.organism = ''
  
  if (globalVariables["mac"] or globalVariables["lin"]):
    self.destroy()
  else:
    controller.show_frame("StartPage")



import cv2
import math

def debugTracking(nbTailPoints, i, firstFrame, x, y, output, outputHeading, frame2, hyperparameters):
  if hyperparameters["debugTracking"]:
    frame2 = cv2.cvtColor(frame2,cv2.COLOR_GRAY2RGB)
    for k in range(0, hyperparameters["nbAnimalsPerWell"]):
      for j in range(0, nbTailPoints):
        x = int(output[k, i-firstFrame][j][0])
        y = int(output[k, i-firstFrame][j][1])
        cv2.circle(frame2, (x, y), 1, (0, 255, 0),   -1)
      x = int(output[k, i-firstFrame][nbTailPoints-1][0]) # it used to be 10 instead of 9 here, not sure why
      y = int(output[k, i-firstFrame][nbTailPoints-1][1]) # it used to be 10 instead of 9 here, not sure why
      cv2.circle(frame2, (x, y), 2, (0, 0, 255),   -1)
      
      x = output[k, i-firstFrame][0][0]
      y = output[k, i-firstFrame][0][1]
      cv2.line(frame2,(int(x),int(y)),(int(x+20*math.cos(outputHeading[k, i-firstFrame])),int(y+20*math.sin(outputHeading[k, i-firstFrame]))), (255,0,0), 3)
      
    cv2.imshow('Frame', frame2)
    cv2.waitKey(0)


import tkinter as tk
from tkinter import font  as tkfont
from tkinter import filedialog
from tkinter import ttk
from tkinter import *
import re
import os
import json
import subprocess
import matplotlib
matplotlib.use("TkAgg")
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
import math
import scipy.io as sio
from readValidationVideo import readValidationVideo
import numpy as np

from vars import getGlobalVariables
globalVariables = getGlobalVariables()

import sys
sys.path.insert(1, './')
sys.path.insert(1, './code/')
sys.path.insert(1, './code/getImage/')
sys.path.insert(1, './code/tracking/')
sys.path.insert(1, './code/tracking/tailTrackingExtremityDetect/')
sys.path.insert(1, './code/tracking/tailTrackingExtremityDetect/findTailExtremete/')
from mainZZ import mainZZ
from getTailExtremityFirstFrame import getTailExtremityFirstFrame
import popUpAlgoFollow

LARGE_FONT= ("Verdana", 12)


class FullScreenApp(object):
    def __init__(self, master, **kwargs):
        self.master=master
        pad=3
        self._geom='200x200+0+0'
        master.geometry("{0}x{1}+0+0".format(
            master.winfo_screenwidth()-pad, master.winfo_screenheight()-pad))
        master.bind('<Escape>',self.toggle_geom)            
    def toggle_geom(self,event):
        geom=self.master.winfo_geometry()
        print(geom,self._geom)
        self.master.geometry(self._geom)
        self._geom=geom


class StartPage(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        label = tk.Label(self, text="Welcome to ZebraZoom!", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)
        
        button0 = tk.Button(self, text="Prepare configuration file for tracking",
                            command=lambda: controller.show_frame("ChooseVideoToCreateConfigFileFor"))#IdentifyHeadCenter")) #ChooseVideoToCreateConfigFileFor"))
        
        button1 = tk.Button(self, text="Run ZebraZoom on a video",
                            command=lambda: controller.show_frame("VideoToAnalyze"))
               
        button3 = tk.Button(self, text="Run ZebraZoom on several videos",
                            command=lambda: controller.show_frame("SeveralVideos"))
        
        button2 = tk.Button(self, text="Visualize ZebraZoom's output",
                            command=lambda: controller.showResultsVisualization())
        
        tk.Label(self, text="", font=controller.title_font).pack()
        button0.pack()
        tk.Label(self, text="", font=controller.title_font).pack()
        button1.pack()

        tk.Label(self, text="", font=controller.title_font).pack()
        button3.pack()

        tk.Label(self, text="", font=controller.title_font).pack()
        button2.pack()
        
        tk.Label(self, text="", font=controller.title_font).pack()
        tk.Button(self, text="Open configuration file folder", command=lambda: controller.openConfigurationFileFolder(controller.homeDirectory)).pack()
        
        tk.Label(self, text="", font=controller.title_font).pack()
        tk.Button(self, text="Open ZebraZoom's output folder", command=lambda: controller.openZZOutputFolder(controller.homeDirectory)).pack()
        
        tk.Label(self, text="", font=controller.title_font).pack()
        tk.Button(self, text="Analyze ZebraZoom's outputs", command=lambda: controller.show_frame("CreateExperimentOrganizationExcel")).pack()
        
        tk.Label(self, text="", font=controller.title_font).pack()
        tk.Button(self, text="Troubleshoot", command=lambda: controller.show_frame("ChooseVideoToTroubleshootSplitVideo")).pack()
        

class SeveralVideos(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        label = tk.Label(self, text="Run ZebraZoom on several videos", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)
            
        button3 = tk.Button(self, text="Run ZebraZoom on an entire folder",
                            command=lambda: controller.show_frame("FolderToAnalyze")).pack()
        
        tk.Label(self, text="", font=controller.title_font).pack()
        
        button4 = tk.Button(self, text="Manual first frame tail extremity for head embedded",
                            command=lambda: controller.show_frame("TailExtremityHE")).pack()
        tk.Label(self, text="", font=controller.title_font).pack()
                            
        tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class VideoToAnalyze(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        label = tk.Label(self, text="Choose video.", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)
        tk.Label(self, text="Look for the video you want to analyze.").pack(side="top", fill="x", pady=10)
        justExtractParams = IntVar()
        noValidationVideo = IntVar()
        button = tk.Button(self, text="Choose file",
                           command=lambda: controller.chooseVideoToAnalyze(justExtractParams.get(), noValidationVideo.get()))
        button.pack()
        
        Checkbutton(self, text="I ran the tracking already, I only want to redo the extraction of parameters.", variable=justExtractParams).pack()
        tk.Label(self, text="", pady=0).pack()
        Checkbutton(self, text="Don't (re)generate a validation video (for speed efficiency).", variable=noValidationVideo).pack()
        tk.Label(self, text="", pady=0).pack()
        
        tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class FolderToAnalyze(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        label = tk.Label(self, text="Choose folder.", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)
        tk.Label(self, text="Look for the folder you want to analyze.").pack(side="top", fill="x", pady=10)
        justExtractParams = IntVar()
        noValidationVideo = IntVar()
        button = tk.Button(self, text="Choose folder",
                           command=lambda: controller.chooseFolderToAnalyze(justExtractParams.get(), noValidationVideo.get()))
        button.pack()
        
        Checkbutton(self, text="I ran the tracking already, I only want to redo the extraction of parameters.", variable=justExtractParams).pack()
        tk.Label(self, text="", pady=0).pack()
        
        Checkbutton(self, text="Don't (re)generate a validation video (for speed efficiency).", variable=noValidationVideo).pack()
        tk.Label(self, text="", pady=0).pack()
        
        tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class TailExtremityHE(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        label = tk.Label(self, text="Choose folder.", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)
        tk.Label(self, text="Look for the folder of videos where you want to manually label tail extremities.").pack(side="top", fill="x", pady=10)
        button = tk.Button(self, text="Choose folder",
                           command=lambda: controller.chooseFolderForTailExtremityHE())
        button.pack()
        tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class ConfigFilePromp(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        label = tk.Label(self, text="Choose configuration file.", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)
        
        button = tk.Button(self, text="Choose file", command=lambda: controller.chooseConfigFile())
        button.pack()
        
        tk.Button(self, text="Go to the start page", command=lambda: controller.show_frame("StartPage")).pack()


class Patience(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        tk.Button(self, text="Launch ZebraZoom on your video(s)", command=lambda: controller.launchZebraZoom()).pack()
        label2 = tk.Label(self, text="After clicking on the button above, please wait for ZebraZoom to run, you can look at the console outside of the GUI to check on the progress of ZebraZoom.")
        label2.pack(side="top", fill="x", pady=10)


class ZZoutro(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        label = tk.Label(self, text="Analysis is finished.")
        label.pack(side="top", fill="x", pady=10)
        button = tk.Button(self, text="Go to the start page",
                           command=lambda: controller.show_frame("StartPage"))
        button.pack()


class ResultsVisualization(tk.Frame):

    def __init__(self, parent, controller):
        nbLines = 15
        curLine = 1
        curCol  = 0
        
        tk.Frame.__init__(self, parent)
        self.controller = controller
        label = tk.Label(self, text="Choose the results you'd like to visualize", font=controller.title_font)
        # label.pack(side="top", fill="x", pady=10)
        label.grid(row=0,column=0)
        
        if not(os.path.exists('./ZZoutput/')):
          os.mkdir('./ZZoutput/')
        
        os.walk('./ZZoutput/')
        for x in sorted(next(os.walk('./ZZoutput'))[1]):
          tk.Button(self, text=x, command=lambda currentResultFolder=x : controller.exploreResultFolder(currentResultFolder)).grid(row=curLine,column=curCol)
          if (curLine > nbLines):
            curLine = 1
            curCol = curCol + 1
          else:
            curLine = curLine + 1
        
        button = tk.Button(self, text="Go to the start page",
                           command=lambda: controller.show_frame("StartPage"))
        button.grid(row=curLine,column=curCol)

        
class ViewParameters(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)
        self.controller = controller
        
        name = self.controller.currentResultFolder
        
        if name != "abc":
          reference = './ZZoutput/'+name+'/results_'+name+'.txt'
          
          if controller.justEnteredViewParameter == 1:
            with open(reference) as ff:
                dataRef = json.load(ff)
            controller.dataRef = dataRef
            controller.justEnteredViewParameter = 0
          else:
            dataRef = controller.dataRef
          
          nbWellsRef  = len(dataRef["wellPoissMouv"])
          
          numWell  = self.controller.numWell
          numPoiss = self.controller.numPoiss
          numMouv  = self.controller.numMouv
          
          nbWells = len(dataRef["wellPoissMouv"])
          if numWell >= nbWells:
            if nbWells == 0:
              numWell = 0
            else:
              numWell = nbWells - 1
          nbPoiss = len(dataRef["wellPoissMouv"][numWell])
          
          if numPoiss >= nbPoiss:
            if nbPoiss == 0:
              numPoiss = 0
            else:
              numPoiss = nbPoiss - 1
          nbMouv  = len(dataRef["wellPoissMouv"][numWell][numPoiss])
          
          if numMouv >= nbMouv:
            if nbMouv == 0:
              numMouv = 0
            else:
              numMouv = nbMouv - 1
          
          # if controller.visualization == 0:
            # label = tk.Label(self, text="Tail Angle Smoothed and min/max amplitudes for well "+str(numWell)+" , fish "+str(numPoiss)+" , bout "+str(numMouv), font=LARGE_FONT)
          # elif controller.visualization == 1:
            # label = tk.Label(self, text="Tail Angle Raw for well "+str(numWell)+" , fish "+str(numPoiss)+" , bout "+str(numMouv), font=LARGE_FONT)
          # else:
            # label = tk.Label(self, text="Body Coordinates for well "+str(numWell)+" , fish "+str(numPoiss)+" , bout "+str(numMouv), font=LARGE_FONT)
          
          # label.grid(row=0,column=7)
          
          label = tk.Label(self, text="    ", font=LARGE_FONT)
          label.grid(row=0,column=6)
          
          ttk.Button(self, text="View video for all wells together", command=lambda: controller.showValidationVideo(-1,0,-1)).grid(row=0,column=1)
          
          label = tk.Label(self, text=name, font=LARGE_FONT)
          label.grid(row=0,column=2)
          
          label = tk.Label(self, text="Well number:")
          label.grid(row=1,column=1)
          e1 = tk.Entry(self)
          e1.insert(0,numWell)
          e1.grid(row=1,column=2)
          tk.Button(self, text="-", command=lambda: controller.printSomeResults(int(e1.get())-1,e2.get(),e3.get())).grid(row=1,column=3)
          tk.Button(self, text="+", command=lambda: controller.printSomeResults(int(e1.get())+1,e2.get(),e3.get())).grid(row=1,column=4)
          
          # ttk.Button(self, text="View video for well "+str(numWell), command=lambda: controller.showValidationVideo(e1.get(),0,-1)).grid(row=2,column=1)
          ttk.Button(self, text="View zoomed video for well "+str(numWell), command=lambda: controller.showValidationVideo(e1.get(),1,-1)).grid(row=2,column=2)
          
          label = tk.Label(self, text="Fish number:")
          label.grid(row=3,column=1)
          e2 = tk.Entry(self)
          e2.insert(0,numPoiss)
          e2.grid(row=3,column=2)
          tk.Button(self, text="-", command=lambda: controller.printSomeResults(e1.get(),int(e2.get())-1,e3.get())).grid(row=3,column=3)
          tk.Button(self, text="+", command=lambda: controller.printSomeResults(e1.get(),int(e2.get())+1,e3.get())).grid(row=3,column=4)
          
          label = tk.Label(self, text="Bout number:")
          label.grid(row=4,column=1)
          e3 = tk.Entry(self)
          e3.insert(0,numMouv)
          e3.grid(row=4,column=2)
          tk.Button(self, text="-", command=lambda: controller.printSomeResults(e1.get(),e2.get(),int(e3.get())-1)).grid(row=4,column=3)
          tk.Button(self, text="+", command=lambda: controller.printSomeResults(e1.get(),e2.get(),int(e3.get())+1)).grid(row=4,column=4)
          
          button1 = ttk.Button(self, text="View bout's angle",
                              command=lambda: controller.printSomeResults(e1.get(),e2.get(),e3.get()))
          button1.grid(row=5,column=2)
          
          
          if (len(dataRef["wellPoissMouv"][numWell][numPoiss]) > 0):
          
            if ("flag" in dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]):
              if (dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["flag"]):
                flagTxt = "UnFlag Movement"
              else:
                flagTxt = "Flag Movement"
            else:
              flagTxt = "Flag Movement"
            
            button3 = tk.Button(self, text=flagTxt, command=lambda: controller.flagMove(e1.get(),e2.get(),e3.get()))
            if flagTxt == "UnFlag Movement":
              button3.configure(bg = 'red')
            button3.grid(row=5,column=4)

          
          prev = ttk.Button(self, text="Previous Bout",
                              command=lambda: controller.printPreviousResults(e1.get(),e2.get(),e3.get(), nbWells, nbPoiss, nbMouv))
          prev.grid(row=6,column=1)
          
          next = ttk.Button(self, text="Next Bout",
                              command=lambda: controller.printNextResults(e1.get(),e2.get(),e3.get(),nbWells,nbPoiss,nbMouv))
          next.grid(row=6,column=2)
          
          tk.Button(self, text="Go to the previous page", command=lambda: controller.show_frame("ResultsVisualization")).grid(row=7,column=1)
          
          tk.Button(self, text="Change Visualization", command=lambda: controller.printSomeResults(e1.get(), e2.get(), e3.get(), True)).grid(row=7,column=2)
          
          # tk.Button(self, text="Refine Parameters Extraction", command=lambda: controller.changeVisualization()).grid(row=8,column=1)
          
          if (controller.superstructmodified == 1):
            button1 = tk.Button(self, text="Save SuperStruct", command=lambda: controller.saveSuperStruct(e1.get(),e2.get(),e3.get()))
            button1.configure(bg = 'orange')
            button1.grid(row=6,column=4)

          if nbMouv > 0:
              
              begMove = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["BoutStart"]
              endMove = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["BoutEnd"]
              
              ttk.Button(self, text="View video for well "+str(numWell), command=lambda: controller.showValidationVideo(e1.get(),0,begMove)).grid(row=2,column=1)
              
              ttk.Button(self, text="View bout's video", command=lambda: controller.showValidationVideo(e1.get(),1,begMove)).grid(row=5,column=1)
              
              
              if controller.visualization == 0 and not("TailAngle_smoothed" in dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]):
                  controller.visualization = 1
              
              if controller.visualization == 1 and not("TailAngle_Raw" in dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]):
                  controller.visualization = 2
              
              if controller.visualization == 2 and not((len(np.unique(dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["HeadX"])) > 1) and (len(np.unique(dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["HeadY"])) > 1)):
                  controller.visualization = 0
              
              if controller.visualization == 0:
                label = tk.Label(self, text="Tail Angle Smoothed and amplitudes for well "+str(numWell)+", fish "+str(numPoiss)+", bout "+str(numMouv), font=LARGE_FONT)
              elif controller.visualization == 1:
                label = tk.Label(self, text="Tail Angle Raw for well "+str(numWell)+", fish "+str(numPoiss)+", bout "+str(numMouv)+"                             ", font=LARGE_FONT)
              else:
                label = tk.Label(self, text="Body Coordinates for well "+str(numWell)+" , fish "+str(numPoiss)+" , bout "+str(numMouv), font=LARGE_FONT)
              label.grid(row=0,column=7)
              
              if controller.visualization == 0:
                
                tailAngleSmoothed = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["TailAngle_smoothed"].copy()
                
                for ind,val in enumerate(tailAngleSmoothed):
                  tailAngleSmoothed[ind]=tailAngleSmoothed[ind]*(180/(math.pi))
                
                if "Bend_Timing" in dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]:
                  freqX = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["Bend_Timing"]
                  freqY = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["Bend_Amplitude"]
                else:
                  freqX = []
                  freqY = []
                if type(freqY)==int or type(freqY)==float:
                  freqY = freqY * (180/(math.pi))
                else:
                  if "Bend_Timing" in dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]:
                    freqX = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["Bend_Timing"].copy()
                    freqY = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["Bend_Amplitude"].copy()
                  else:
                    freqX = []
                    freqY = []
                  for ind,val in enumerate(freqY):
                    freqY[ind]=freqY[ind]*(180/(math.pi))
                fx = [begMove]
                fy = [0]
                if (type(freqX) is int) or (type(freqX) is float):
                  freqX = [freqX]
                  freqY = [freqY]
                for idx,x in enumerate(freqX):
                  idx2 = idx - 1
                  fx.append(freqX[idx2] - 1 + begMove)
                  fx.append(freqX[idx2] - 1 + begMove)
                  fx.append(freqX[idx2] - 1 + begMove)
                  fy.append(0)
                  fy.append(freqY[idx2])
                  fy.append(0)
                
                f = Figure(figsize=(5,5), dpi=100)
                a = f.add_subplot(111)
                
                if len(tailAngleSmoothed):
                  a.plot([i for i in range(begMove,endMove+1)],tailAngleSmoothed)
                  a.plot(fx,fy)
                  a.plot([i for i in range(begMove,endMove+1)],[0 for i in range(0,len(tailAngleSmoothed))])
                
              elif controller.visualization == 1:
                
                tailAngleSmoothed = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["TailAngle_Raw"].copy()
                for ind,val in enumerate(tailAngleSmoothed):
                  tailAngleSmoothed[ind]=tailAngleSmoothed[ind]*(180/(math.pi))
              
                f = Figure(figsize=(5,5), dpi=100)
                a = f.add_subplot(111)
              
                a.plot([i for i in range(begMove,endMove+1)],tailAngleSmoothed)
                a.plot([i for i in range(begMove,endMove+1)],[0 for i in range(0,len(tailAngleSmoothed))])
                
              else:
                
                headX = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["HeadX"].copy()
                headY = dataRef["wellPoissMouv"][numWell][numPoiss][numMouv]["HeadY"].copy()
                
                f = Figure(figsize=(5,5), dpi=100)
                a = f.add_subplot(111)
                
                a.plot(headX, headY)
              
              canvas = FigureCanvasTkAgg(f, self)
              canvas.draw()
              canvas.get_tk_widget().grid(row=1,column=7,rowspan=7)
              
              # toolbar = NavigationToolbar2TkAgg(canvas, self)
              # toolbar.update()
              # canvas._tkcanvas.grid(row=8,column=6,rowspan=1)

          else:
          
              tailAngleSmoothed = [i for i in range(0,1)]
          
              f = Figure(figsize=(5,5), dpi=100)
              a = f.add_subplot(111)
              
              a.plot([i for i in range(0,len(tailAngleSmoothed))],tailAngleSmoothed)

              canvas = FigureCanvasTkAgg(f, self)
              canvas.draw()
              canvas.get_tk_widget().grid(row=1,column=7,rowspan=7)
              
              canvas = Canvas(self)
              canvas.create_text(100,10, text="No bout detected for well "+str(numWell))
              canvas.grid(row=1,column=7,rowspan=7)


import numpy as np
from scipy.interpolate import UnivariateSpline
import math

def getGlobalParameters(curbout, fps, pixelSize):

  BoutDuration = (curbout["BoutEnd"] - curbout["BoutStart"] + 1) / fps
  
  NumberOfOscillations = len(curbout["Bend_Timing"]) / 2
  
  TotalDistance = 0 
  posX = curbout["HeadX"]
  posX = [posX[4*i] for i in range(0, int(len(posX)/4))]
  posY = curbout["HeadY"]
  posY = [posY[4*i] for i in range(0, int(len(posY)/4))]
  for j in range(1, len(posX)-1):
      TotalDistance = TotalDistance + math.sqrt((posX[j+1] - posX[j])**2 + (posY[j+1] - posY[j])**2)
  TotalDistance = TotalDistance * pixelSize

  Speed = TotalDistance / BoutDuration
  
  meanTBF = NumberOfOscillations / BoutDuration
  
  maxAmplitude = max([abs(ta) for ta in curbout["TailAngle_smoothed"]])
  
  firstBendTime = curbout["Bend_Timing"][0]
  
  firstBendAmplitude = abs(curbout["Bend_Amplitude"][0])
  
  return [BoutDuration, TotalDistance, Speed, NumberOfOscillations, meanTBF, maxAmplitude, posY[0], posY[len(posY)-1], np.mean(posY), firstBendTime, firstBendAmplitude]



import numpy as np
import cv2
import math
import json
import popUpAlgoFollow
import random

def createValidationVideo(videoPath, superStruct, hyperparameters):

  firstFrame                  = hyperparameters["firstFrame"]
  lastFrame                   = hyperparameters["lastFrame"]
  plotOnlyOneTailPointForVisu = hyperparameters["plotOnlyOneTailPointForVisu"]
  trackingPointSizeDisplay    = hyperparameters["trackingPointSizeDisplay"]
  
  cap = cv2.VideoCapture(videoPath)
  if (cap.isOpened()== False): 
    print("Error opening video stream or file")

  frame_width  = int(cap.get(3))
  frame_height = int(cap.get(4))
  nbFrames     = int(cap.get(7))
  
  infoFrame = [[]] * nbFrames
  infoWells = []
  
  for i in range(0, len(superStruct["wellPositions"])):
    x = superStruct["wellPositions"][i]["topLeftX"]
    y = superStruct["wellPositions"][i]["topLeftY"]
    infoWells.append([x, y])
  
  for i in range(0, len(superStruct["wellPoissMouv"])):
    colorModifTab = [{"red": random.randrange(255), "green": random.randrange(255), "blue": random.randrange(255)} for i in range(1, hyperparameters["nbAnimalsPerWell"])]
    colorModifTab.insert(0, {"red": 0, "green": 0, "blue": 0})
    # colorModifTab.insert(0, {"red": 0, "green": 0, "blue": 0})
    for j in range(0, len(superStruct["wellPoissMouv"][i])):
      for k in range(0, len(superStruct["wellPoissMouv"][i][j])):
      
        BoutStart = superStruct["wellPoissMouv"][i][j][k]["BoutStart"]
        BoutEnd = superStruct["wellPoissMouv"][i][j][k]["BoutEnd"]
        mouvLength = len(superStruct["wellPoissMouv"][i][j][k]["HeadX"])
        if hyperparameters['extractAdvanceZebraParameters']:
          Bend_TimingAbsolute = superStruct["wellPoissMouv"][i][j][k]["Bend_TimingAbsolute"]
        else:
          Bend_TimingAbsolute = []
        
        anglePointsBefore = []
        anglePointsBaseBefore = []
        x0 = superStruct["wellPoissMouv"][i][j][k]["HeadX"][0]
        y0 = superStruct["wellPoissMouv"][i][j][k]["HeadY"][0]
        for l in range(0,mouvLength):
          if BoutStart + l < nbFrames:
            x = superStruct["wellPoissMouv"][i][j][k]["HeadX"][l]
            y = superStruct["wellPoissMouv"][i][j][k]["HeadY"][l]
            Heading = superStruct["wellPoissMouv"][i][j][k]["Heading"][l]
            
            x = x + infoWells[i][0]
            y = y + infoWells[i][1]
            dataToPlot = {}
            dataToPlot["x"]       = x
            dataToPlot["y"]       = y
            dataToPlot["size"]    = trackingPointSizeDisplay + 2
            dataToPlot["Heading"] = Heading
            dataToPlot["numMouv"] = k+1
            dataToPlot["numWell"] = i
            
            dataToPlot["red"]     = 255
            dataToPlot["green"]   = 0
            dataToPlot["blue"]    = 0
            
            t = infoFrame[BoutStart + l]
            t2 = t.copy()
            t2.append(dataToPlot)
            infoFrame[BoutStart + l] = t2
            
            # Points along the Tail
            nbPointsToPlot   = 0
            firstPointToPlot = 0
            if plotOnlyOneTailPointForVisu:
              firstPointToPlot = len(superStruct["wellPoissMouv"][i][j][k]["TailX_VideoReferential"][l]) - 1
              nbPointsToPlot   = len(superStruct["wellPoissMouv"][i][j][k]["TailX_VideoReferential"][l])
            else:
              firstPointToPlot = 0
              nbPointsToPlot = len(superStruct["wellPoissMouv"][i][j][k]["TailX_VideoReferential"][l])
            
            for m in range(firstPointToPlot, nbPointsToPlot):
              tailX = superStruct["wellPoissMouv"][i][j][k]["TailX_VideoReferential"][l][m]
              tailY = superStruct["wellPoissMouv"][i][j][k]["TailY_VideoReferential"][l][m]
              tailX = tailX + infoWells[i][0]
              tailY = tailY + infoWells[i][1]
              size = 1
              dataToPlot      = {}
              dataToPlot["x"] = tailX
              dataToPlot["y"] = tailY
              if plotOnlyOneTailPointForVisu:
                if (Bend_TimingAbsolute.count(BoutStart+l+1) != 0):
                  dataToPlot["size"] = trackingPointSizeDisplay + 5
                  dataToPlot["red"]   = 0
                  dataToPlot["green"] = 0
                  dataToPlot["blue"]  = 255
                else:
                  dataToPlot["size"] = trackingPointSizeDisplay + 2
                  dataToPlot["red"]   = 0
                  dataToPlot["green"] = 255
                  dataToPlot["blue"]  = 0
              else:
                if (m == (nbPointsToPlot-1)):
                  if (Bend_TimingAbsolute.count(BoutStart+l+1) != 0):
                    dataToPlot["size"] = trackingPointSizeDisplay + 5
                    dataToPlot["red"]   = 0
                    dataToPlot["green"] = 0
                    dataToPlot["blue"]  = 255
                  else:
                    dataToPlot["size"] = trackingPointSizeDisplay + 2
                    dataToPlot["red"]   = 0
                    dataToPlot["green"] = 255
                    dataToPlot["blue"]  = 0
                else:
                  dataToPlot["size"] = trackingPointSizeDisplay;
                  dataToPlot["red"]   = 0   + colorModifTab[j]["red"]
                  dataToPlot["green"] = 255 - colorModifTab[j]["green"]
                  dataToPlot["blue"]  = 0   + colorModifTab[j]["blue"]
              dataToPlot["Heading"] = -10
              
              t = infoFrame[BoutStart + l]
              t2 = t.copy()
              t2.append(dataToPlot)
              infoFrame[BoutStart + l] = t2

  # Going through the video and printing stuff on it.
  outputName = hyperparameters["outputFolder"] + hyperparameters["videoName"] + '/' + hyperparameters["videoName"] + '.avi'
  out = cv2.VideoWriter(outputName,cv2.VideoWriter_fourcc('M','J','P','G'), 10, (frame_width,frame_height))

  cap.set(1, firstFrame)
  
  for l in range(firstFrame, lastFrame):
  
    if l < nbFrames:
      # cap.set(1, l)
      ret, frame = cap.read()
      
      for i in range(0, len(infoFrame[l])):
        x = infoFrame[l][i]["x"]
        y = infoFrame[l][i]["y"]
        size  = infoFrame[l][i]["size"]
        red   = infoFrame[l][i]["red"]
        green = infoFrame[l][i]["green"]
        blue  = infoFrame[l][i]["blue"]
        if (infoFrame[l][i]["Heading"] != -10):
          heading = infoFrame[l][i]["Heading"]
          if hyperparameters["validationVideoPlotHeading"]:
            if hyperparameters["debugValidationVideoHeading"] == 0:
              cv2.line(frame,(int(x),int(y)),(int(x+20*math.cos(heading)),int(y+20*math.sin(heading))), (255,0,0), trackingPointSizeDisplay+1)
            else:
              cv2.line(frame,(int(x),int(y)),(int(x-250*math.cos(heading)),int(y-250*math.sin(heading))), (255,0,0), trackingPointSizeDisplay+1)
          
          font = cv2.FONT_HERSHEY_SIMPLEX
          numMouv = infoFrame[l][i]["numMouv"]
          numWell = infoFrame[l][i]["numWell"]
          cv2.putText(frame,str(numMouv),(15+infoWells[numWell][0],25+infoWells[numWell][1]),font,1,(255,255,255))
          
          # CvFont font;
          # cvInitFont( &font, CV_FONT_HERSHEY_SIMPLEX,0.7,0.7,0,2,8);
          # std::ostringstream out;
          # out << infoFrame[l][i]["numMouv"];
          # string str = out.str();
          # size_t size = str.size() + 1;
          # char * buffer = new char[ size ];
          # strncpy( buffer, str.c_str(), size );
          # int numWell = (int) infoFrame[l][i]["numWell"];
          # cvRectangle(dst,cvPoint((int)infoWells[numWell][0],(int)infoWells[numWell][1]),cvPoint(15 + (int)infoWells[numWell][0], 25 + (int)infoWells[numWell][1]),CV_RGB(0,0,255),-1);
          # cvPutText(dst,buffer,cvPoint((int)infoWells[numWell][0],20+(int)infoWells[numWell][1]),&font,CV_RGB(255,255,255));

        cv2.circle(frame, (int(x),int(y)), size, (red,green,blue), -1)
        
      # cv2.imshow('Frame', frame)
      # cv2.waitKey(10)
      out.write(frame)
      
      # l = l + 1
  
  out.release()
  
  if (hyperparameters["freqAlgoPosFollow"] != 0):
    print("Create Validation Video")
  if hyperparameters["popUpAlgoFollow"]:
    popUpAlgoFollow.prepend("Create Validation Video")

  return infoFrame
